#!/usr/bin/perl
# train the svm

use Getopt::Long;

#
# read command line and check its correctness
#
$options = "Usage: ./TRAIN.pl <options>
  REQUIRED INPUTS:
   -t training sequences (required)
   -f feature list (required)
   -o output prefix (required)

  EXTRA SETTINGS:
   -s feature selection (y or n, default y)
   -v cross validation split size (integer bigger than 2, default 3)
   -k kernel: linear or radial (l or r, default r)

  COMPUTATIONAL SETTINGS (won't affect results):
   -p program directory (for batch running on a cluster)
      set to be the directory that contains all the scripts (default: ./)\n";

$train = "";
$feats = "";
$out = "";
$fsel = "y";
$repeat = 3;
$flanklen = 3;
$kernel = "r";
$progdir = ".";

GetOptions('t=s' => \$train,
           'f=s' => \$feats,
           'o=s' => \$out,
           's=s' => \$fsel,
           'v=i' => \$repeat,
           'k=s' => \$kernel,
           'p=s' => \$progdir) || die "$options\n";

if( $train eq "" ){
	print "Error: set training sequences\n$options\n";
	exit;
}
if( $feats eq "" ){
	print "Error: set feature list\n$options\n";
	exit;
}
if( $out eq "" ){
	print "Error: set output prefix\n$options\n";
	exit;
}

printf("\nInput data: %s\nFeatures: %s\nOutput: %s\n", $train, $feats, $out);
if( $fsel ne "y" ){
	if( $fsel ne "n" ){
		print "Error: set feature selection to be y or n\n";
		exit;
	}
	printf("Not using feature selection\n");
}
if( $repeat < 2 ){
	print "Error: set repeat to be at least 2\n";
	exit;
}
printf("Running %i-way cross validation\n", $repeat);
if( $kernel ne "r" ){
	if($kernel ne "l" ){
		print "Error: set kernel to be l or r\n";
		exit;
	}
	printf("Using linear kernel\n");
}
else{
	printf("Using radial kernel\n");
}

#
# parameterize the test sequences
#
print "\nParameterizing the training data\n";
system("$progdir/parameterize.pl $train $out.trainvec $flanklen $feats -1 $out.log");
&check_stat("parameterize.pl");
if( $fsel eq "y" ){
	system("$progdir/select_features.pl $out.trainvec $out.vecvals");
	system("$progdir/shorten_features.pl $out.trainvec $out.vecvals $out.shortvec");
}
if( $fsel eq "n" ){
	system("cp $out.trainvec $out.shortvec");
}


#
# search for optimum weights
# after finishing, read them in and define options
#
print "\nRunning cross validation\n";
system("$progdir/search_svm_params.pl -t $out.shortvec -o $out.wts -l $out.log -n $repeat -p $progdir -k $kernel");
&check_stat("search_svm_params.pl");
#if( $kernel eq "l" ){
#	system("$progdir/search_svm_params.pl $out.shortvec $out.wts $out.log $repeat $progdir");
#	&check_stat("search_svm_params.pl");
#}
#else{
#	system("$progdir/search_svm_params_radial.pl $out.shortvec $out.wts $out.log $repeat $progdir");
#	&check_stat("search_svm_params_radial.pl");
#}

open(PARS,"$out.wts") || die "Error: can't open the weights for some reason\n";
$line = <PARS>;
close(PARS);
@lpts = split(' ',$line);
$C1 = $lpts[9];
$C2 = $lpts[11];

#
# train the svm
#
print "\nTraining the SVM\n";
if( $kernel eq "l" ){
	$svmopts = "-t 0 -w1 $C1 -w-1 $C2 -q";
}
else{
	$svmopts = "-c $C1 -g $C2 -q";
}
system("$progdir/svm-train $svmopts $out.shortvec $out.model");



system("rm $out.log");
print "\nFinished!\n\n";
exit;

# function that checks $out.log for success of script
# $_[0] is name of script last run
sub check_stat{
	my $line;
	$line = `tail -1 $out.log`;
	chomp($line);
	if( $line ne "success" ){
		print "Error in script $_[0]; exiting program...\n";
		exit;
	}
}

















